@@ -5,7 +5,7 @@
     Author:        Jan Wielemaker and Peter Ludemann
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2022, SWI-Prolog Solutions b.v.
+    Copyright (c)  2022-2023, SWI-Prolog Solutions b.v.
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -39,12 +39,14 @@
 :- module(test_cpp,
 	  [ test_cpp/0
 	  ]).
+:- use_module(library(debug)).
+:- use_module(library(lists)).
+:- use_module(library(apply)).
+:- autoload(library(aggregate)).
+:- use_module(library(plunit)).
 
 :- encoding(utf8).
 
-:- use_module(library(plunit)).
-:- use_module(test_common).
-
 :- use_foreign_library(foreign(test_cpp)).
 
 test_cpp :-
@@ -80,10 +82,15 @@ test(hello_query, fail) :-
 
 test(as_string, S == "foo") :-
     atom_to_string(foo, S).
-test(as_string, S == "ä¸\u0096ç\u0095\u008Cå\u009B\u009B") :-
-    atom_to_string(世界四, S).
 test(as_string, S = "foo(bar)") :-
     term_to_string(foo(bar), S).
+
+% Note: atom_to_string/2 and term_to_string/2 translate the data
+% to a UTF-8 string.  We currenly do not support encoding for
+% PlTerm.unify_string(), so we get as result the byte encoding
+% of the UTF8 data.
+test(as_string, S == "ä¸\u0096ç\u0095\u008Cå\u009B\u009B") :-
+    atom_to_string(世界四, S).
 test(as_string, S = "hello(ä¸\u0096ç\u0095\u008Cå\u009B\u009B)") :-
     term_to_string(hello(世界四), S).
 
@@ -112,6 +119,16 @@ test(average_3, Average =:= Expected) :-
 test(hello_0, Out == "hello world\n") :-
     with_output_to(string(Out), hello).
 
+call_cut_test :-
+    setup_call_cleanup(true,
+		       between(1, 5, _X),
+		       atom_codes(_,_)).
+
+test(call_cut, error(existence_error(procedure,call_cut_test/0))) :-
+    % This tests that an error in ~PlQuery() is handled properly
+    % See discussion: https://github.com/SWI-Prolog/packages-cpp/pull/27
+    call_cut("call_cut_test").
+
 test(term_1, Term = hello(world)) :-
     term(Term).
 
@@ -198,12 +215,27 @@ cpp_call(Goal, Flags) :-
 
 test(square_roots_2a, Result == [0.0, 1.0, 1.4142135623730951, 1.7320508075688772, 2.0]) :-
     square_roots(5, Result).
+
+:- meta_predicate with_small_stacks(+, 0).
+with_small_stacks(Free, Goal) :-
+    garbage_collect,
+    statistics(globalused, G),
+    statistics(trailused, T),
+    statistics(localused, L),
+    NewLimit is G+L+T+Free,
+    current_prolog_flag(stack_limit, Old),
+    setup_call_cleanup(
+	set_prolog_flag(stack_limit, NewLimit),
+	Goal,
+	set_prolog_flag(stack_limit, Old)).
+
 test(square_roots_2b, error(resource_error(stack))) :-
-    square_roots(1000000000, _).
+    with_small_stacks(5 000 000, % 400 000 seems to be about the smallest allowed value
+		      square_roots(1000000000, _)).
 
 test(malloc) :-
-    malloc(1000, Result), % smoke test
-    free(Result).
+    malloc_new(1000, Result), % smoke test
+    free_delete(Result).
 
 :- if(\+ current_prolog_flag(asan, true)).
 too_big_alloc_request(Request) :-
@@ -230,31 +262,23 @@ too_many_bits_alloc_request(Request) :-
 
 :- endif.
 
-test(malloc) :-
+test(malloc, error(resource_error(memory))) :-
     too_big_alloc_request(Request),
-    malloc(Request, Result),
-    assertion(Result == 0),
-    free(Result).
+    malloc_new(Request, Result).
 
 :- if(current_prolog_flag(bounded,false)).
 
 test(malloc) :-
     too_many_bits_alloc_request(Request),
-    catch( ( malloc(Request, Result),
-	     free(Result)
+    catch( ( malloc_new(Request, Result),
+	     free_delete(Result)
 	   ),
 	   error(E,_), true),
-    assertion(memberchk(E, [representation_error(_),
+    assertion(memberchk(E, [representation_error(uint64_t),
 			    type_error(integer,_)])).
 
 :- endif.
-:- endif.                    % \+current_prolog_flag(asan, true)
 
-test(new_chars_1) :-
-    new_chars(1000, Result), % smoke test
-    delete_chars(Result).
-
-:- if(\+ current_prolog_flag(asan, true)).
 % ASAN has maximum 0x10000000000
 %   see ASAN_OPTIONS=allocator_may_return_null=1:soft_rss_limit_mb=...:hard_rss_limit_mb=...
 % https://github.com/google/sanitizers/issues/295
@@ -265,8 +289,6 @@ test(new_chars_2, error(resource_error(memory))) :-
     new_chars(Request, Result),
     delete_chars(Result).
 
-:- endif.
-
 :- if(current_prolog_flag(bounded,false)).
 
 test(new_chars_3) :-
@@ -279,6 +301,11 @@ test(new_chars_3) :-
 			    type_error(integer,_)])).
 
 :- endif.
+:- endif.
+
+test(new_chars_1) :-
+    new_chars(1000, Result), % smoke test
+    delete_chars(Result).
 
 test(name_arity_1, Out == "name = foo, arity = 2\n") :-
     name_arity(foo(bar,zot), Out).
@@ -511,3 +538,22 @@ test(error_term, [error(domain_error(footype,qqsv("ABC")),_)]) :-
 
 w_atom_cpp(Atom, String) :-
     with_output_to(string(String), w_atom_cpp_(current_output, Atom)).
+
+%!  query_flag(?Name, ?Bit)
+%
+%   Flags for  PL_open_query().  Check  with SWI-Prolog.h.   Same code
+%   appears   in  test_ffi.pl.    This  is   duplicated  to   simplify
+%   installation of these tests in the binary version.
+
+% query_flag(debug,		I) => I =0x0001.
+% query_flag(deterministic,	I) => I =0x0100.
+query_flag(normal,		I) => I =0x0002.
+query_flag(nodebug,		I) => I =0x0004.
+query_flag(catch_exception,	I) => I =0x0008.
+query_flag(pass_exception,	I) => I =0x0010.
+query_flag(allow_yield,		I) => I =0x0020.
+query_flag(ext_status,		I) => I =0x0040.
+
+query_flags(Flags, CombinedFlag) :-
+    maplist(query_flag, Flags, Ints),
+    aggregate_all(sum(I), member(I, Ints), CombinedFlag).
