@@ -46,7 +46,6 @@
 #define O_SHAREDLIBRARY 1
 
 #ifdef SWI
-#define PL_ARITY_AS_SIZE 1
 #include <SWI-Stream.h>			/* get intptr_t, etc first */
 #include <SWI-Prolog.h>
 #endif
@@ -264,9 +263,9 @@ initPrologConstants()
 { ATOM_append			= AtomFromString("append");
   ATOM_argument			= AtomFromString("argument");
   ATOM_argument_count		= AtomFromString("argument_count");
-  ATOM_assign		        = AtomFromString(":=");
+  ATOM_assign			= AtomFromString(":=");
   ATOM_badIntegerReference	= AtomFromString("bad_integer_reference");
-  ATOM_badList		        = AtomFromString("bad_list");
+  ATOM_badList			= AtomFromString("bad_list");
   ATOM_badObjectDescription	= AtomFromString("bad_object_description");
   ATOM_badReference		= AtomFromString("bad_reference");
   ATOM_badSelector		= AtomFromString("bad_selector");
@@ -288,10 +287,10 @@ initPrologConstants()
   ATOM_named_argument		= AtomFromString("named_argument");
   ATOM_named_reference		= AtomFromString("named_reference");
   ATOM_new			= AtomFromString("new");
-  ATOM_object		        = AtomFromString("object");
-  ATOM_open		        = AtomFromString("open");
+  ATOM_object			= AtomFromString("object");
+  ATOM_open			= AtomFromString("open");
   ATOM_pce			= AtomFromString("pce");
-  ATOM_permission_error	        = AtomFromString("permission_error");
+  ATOM_permission_error		= AtomFromString("permission_error");
   ATOM_procedure		= AtomFromString("procedure");
   ATOM_proper_list		= AtomFromString("proper_list");
   ATOM_read			= AtomFromString("read");
@@ -1864,7 +1863,7 @@ get_pce_class(term_t t, PceClass *cl)
 
       if ( class )
       { *cl = class;
-        return TRUE;
+	return TRUE;
       }
     }
 
@@ -1909,9 +1908,7 @@ put_prolog_argument(PceGoal g, term_t t, PceType type, term_t f)
   term_value_t val;
 					/* --> :prolog */
   if ( pceIncludesHostDataType(type, ClassProlog) )
-  { PutTerm(t, f);
-    return TRUE;
-  }
+    return PutTerm(t, f);
 
   switch(PL_get_term_value(f, &val))
   { case PL_ATOM:
@@ -1926,9 +1923,7 @@ put_prolog_argument(PceGoal g, term_t t, PceType type, term_t f)
       break;
     case PL_FLOAT:
       if ( pceCheckFloatType(type, val.f) )
-      { PutTerm(t, f);
-	return TRUE;
-      }
+	return PutTerm(t, f);
       break;
     case PL_TERM:
       if ( val.t.name == ATOM_ref && val.t.arity == 1 )
@@ -1941,9 +1936,11 @@ put_prolog_argument(PceGoal g, term_t t, PceType type, term_t f)
 
 	if ( (obj2 = pceCheckType(g, type, obj)) )
 	{ if ( obj2 == obj )
-	    PutTerm(t, f);
-	  else
-	    put_object(t, obj2);
+	  { if ( !PutTerm(t, f) )
+	      return FALSE;
+	  } else
+	  { put_object(t, obj2);
+	  }
 
 	  return TRUE;
 	}
@@ -2475,7 +2472,7 @@ put_object(term_t t, PceObject obj)
       term_t t2;
 
       return ( (t2 = PL_new_term_ref()) &&
-	       PL_put_integer(t2, value.integer) &&
+	       PL_put_int64(t2, value.integer) &&
 	       PL_cons_functor(t, FUNCTOR_ref1, t2) );
 #endif
       break;
@@ -2500,14 +2497,9 @@ put_object(term_t t, PceObject obj)
       break;
     }
     case PCE_HOSTDATA:
-    { PutTerm(t, getTermHandle(obj));	/* TBD: Use saved handle */
-      return TRUE;
-      break;
-    }
+      return PutTerm(t, getTermHandle(obj));	/* TBD: Use saved handle */
     case PCE_INTEGER:
-      return PL_put_integer(t, value.integer);
-
-      break;
+      return PL_put_int64(t, value.integer);
     case PCE_NAME:
       { PceITFSymbol symbol = value.itf_symbol;
 
@@ -3154,7 +3146,7 @@ prof_activate(int active)
 
   memset(&hooks, 0, sizeof(hooks));
   if ( active )
-  { hooks.call   = (void*)PL_prof_call;
+  { hooks.call   = (void*(*)())PL_prof_call;
     hooks.exit   = PL_prof_exit;
     hooks.handle = &pceProfType;
   }
